# encoding: UTF-8
#--
#
#################################################################################
# Copyright 2010-2013 Free Software's Seed, CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de la gestión de agente. Este controlador es utilizado desde las secciones:
# * Sección administración: se utiliza para crear agentes y asignarles usuarios.
# * Sección agentes: se utiliza para seleccionar el agente

class AgenteController < ApplicationController
  # en administracion: se redirecciona por defecto a ordenado
  def index
    redirect_to :action => 'ordenado'
  end

  # en administracion: lista las agentes que hay en el sistema
  def listado
    #session[:agente_asc_desc] ||= "ASC"
    #session[:agente_orden] ||= "nombre"
    @agente = params[:id] ?  Agente.find(params[:id]) : nil
    @agentes = @paginado =  Agente.paginate( :include => "pais", :order => (session[:agente_orden] + " " + session[:agente_asc_desc] ), :page => params[:page], :per_page => (session[:por_pagina]) )
  end

  # en administracion o en agentes: establece los parametros de ordenación
  def ordenado
    session[:agente_asc_desc] = params[:asc_desc] ? params[:asc_desc] : "ASC"   
    session[:agente_orden] = params[:orden] ? params[:orden] : "nombre" 
    redirect_to :action => "listado" if params[:seccion] == "administracion"
    redirect_to :action => "listado_usuario" if params[:seccion] == "agentes"
  end
  
  # en administracion: prepara el formulario de edición o creación
  def editar_nuevo
    @agente = params[:id] ?  Agente.find(params[:id]) : nil
    @paises = Pais.order(:nombre).all.collect {|p| [p.nombre, p.id]}
    @monedas = Moneda.order(:nombre).collect {|p| [p.nombre, p.id]}
    render :partial => "formulario"
  end

  # en administracion: modifica o crea
  def modificar_crear
    @agente = params[:id] ?  Agente.find(params[:id]) : Agente.new 
    @agente.update_attributes params[:agente]
    msg @agente
    redirect_to :action => "listado"
  end

  # en administracion: elimina
  def eliminar
    @agente = Agente.find(params[:id])
    @agente.destroy
    msg_eliminar @agente
    redirect_to :action => 'listado'
  end

  # en agentes: lista los agentes asociados al usuario declarado en la sessión
  def listado_usuario
    @agentes = @usuario_identificado.agente.all(:conditions => {:socia_local => false})
  end

end
