# encoding: UTF-8
#--
#
#################################################################################
# Copyright 2010-2013 Free Software's Seed, CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de la gestión de los documentos.
# Este controlador es utilizado desde las secciones:
# * Sección proyectos: se utiliza para listar, comentar, subir y descargar documentos
# * Sección financiaciones: se utiliza para listar, comentar, subir y descargar documentos
class DocumentoController < ApplicationController

  # Solo elimina cuando el documento pertenece
  before_filter :espacio_seleccionado, :except => [ :index, :seleccionar_espacio ]
  before_filter :autorizar_documento, :only => [ :eliminar, :modificar_crear, :editar_nuevo, :asociar_o_nuevo, :asociar_documentos, :asociar ]


  # Este metodo es solamente para tener una variable que utilizamos mucho en muchos metodos
  def espacio_seleccionado
    @espacio_id = session[:espacio_proyecto_seleccionado] if params[:seccion] == "proyectos"
    @espacio_id = session[:espacio_agente_seleccionado] if params[:seccion] == "agentes"
    @espacio_id = session[:espacio_seleccionado] if params[:seccion] == "documentos"
    @espacio_id = Espacio.find_by_nombre("Plantillas exportación").id  if params[:seccion] == "administracion"
  end


  # Este metodo controla si se puede actualizar un documento dado
  def autorizar_documento
    @documento = Documento.find_by_id(params[:id])
    # Si el documento existe y (tiene espacio y es un documento original) o (no tiene espacio y es un objeto relacionado)
    if @documento && ( (@documento.espacio.size > 0 && @documento.espacio.first.id == @espacio_id) || (@documento.espacio.size == 0 && params[:objeto_id]))
      # Permitimos modificar y borrar solo al propietario del documento y al coordinador.
      if @documento.usuario_id && @documento.usuario_id != @usuario_identificado.id 
        render (:update) {|page|  page.mensaje_informacion params[:update], _("No tiene permisos para borrar o modificar el documento."), :tipo_mensaje => "mensajefallo"} unless @proyecto and @proyecto.usuario_x_proyecto.find_by_usuario_id( @usuario_identificado.id ).rol == "coordinador" # con la excepcion de si es coordinador de proyecto.
      end
    # Si no existe, autoriza segun el espacio y el usuario
    elsif @espacio_id && !(Espacio.find_by_id(@espacio_id).escritura_permitida @usuario_identificado)
      #debugger      
      # Permitirmos anadir documentos solo a los usuarios incluidos en un determinado espacio
      render (:update) {|page|  page.mensaje_informacion params[:update], _("No tiene permisos para añadir documentos en este espacio."), :tipo_mensaje => "mensajefallo"}
    end
  end

	# en proyectos y en financiación: se redirecciona por defecto a ordenado
  def index
    session[:espacio_seleccionado] = nil  if params[:seccion] == "documentos"
    session[:espacio_seleccionado] = Espacio.find_by_nombre("Plantillas exportación").id  if params[:seccion] == "administracion"
    session[:espacio_proyecto_seleccionado] = @proyecto.espacio.id if params[:seccion] == "proyectos"
    session[:espacio_agente_seleccionado] = @agente.espacio.id if params[:seccion] == "agentes"	
    redirect_to :action => "ordenado"
  end

	# en proyectos y en financiación: establece los parametros de ordenación
  def ordenado
    session[:documento_asc_desc] = params[:asc_desc] ? params[:asc_desc] : "ASC" 
    session[:documento_orden] = params[:orden] ? params[:orden] : "adjunto_file_name" 
    redirect_to :action => "listado"
  end


  def seleccionar_espacio
    espacio = Espacio.find_by_id params[:id]
    #if params[:id].nil? or (espacio and (espacio.ocultar != true or espacio.usuario.include? @usuario_identificado))
    if params[:id].nil? or (espacio and !espacio.usuario_no_permitido(@usuario_identificado))
      session[:espacio_seleccionado] = params[:id] 
    end if params[:seccion] == "documentos"
    session[:espacio_proyecto_seleccionado] = params[:id] if params[:seccion] == "proyectos"
    session[:espacio_agente_seleccionado] = params[:id] if params[:seccion] == "agentes"
    session[:espacio_seleccionado] = Espacio.find_by_nombre("Plantillas exportación").id if params[:seccion] == "administracion"
    redirect_to :action => :listado
  end

  def implementador
    @ruta = [{:nombre => _("Raíz")}, {:nombre => _("Implementadores"), :action => 'implementador'}]
    espacios = @usuario_identificado.agente(:order => 'nombre')
    @espacios = espacios.inject([]) {|m,a| m.push({:nombre => a.nombre, :id => a.espacio.id, :action => 'seleccionar_espacio'}) }
    render 'listado'
  end

  def listado
    obtiene_espacios_y_documentos
  end

#--
# ACCIONES COMUNES DE DOCUMENTOS: crear, editar, elminar, descargar
#++

	# en proyectos y en financiación: prepara el formulario de edición o creación
  def editar_nuevo
    if params[:documento]
      # Todo esto hay que hacerlo por el iframe de subida con ajax
      params[:tipo] ||= params[:url][:tipo] if params[:url]
      params[:objeto_id] ||= params[:url][:objeto_id] if params[:url]
      params[:i] ||= params[:url][:i] if params[:url]
      modificar_crear
    else
      @documento = params[:id] ?  Documento.find(params[:id]) : Documento.new
      carga_espacios_etiquetas
      espacio =  Espacio.find(@espacio_id) if @espacio_id
      render :update do |page|
        # Si se estamos navegando por espacios no dejamos que se pueda tocar espacios de plantillas
        #if params[:menu] == "documentos_generales" and (espacio.definicion_espacio_proyecto_id or espacio.proyecto_id or 
        #   espacio.definicion_espacio_agente_id or (espacio.agente_id && !espacio.agente.socia_local) or !espacio.modificable ) 
        #  page.mensaje_informacion params[:update], "No se puede incluir documentación para espacios de proyectos, o de agentes, o de administración, desde esta sección.", :tipo_mensaje => "mensajefallo"
        #else
          page.formulario :partial => "formulario", :update => params[:update]  unless params[:pisa_formulario]
          page.replace 'formulariocontenedor', :partial => "formulario", :locals => { :update => params[:update], :update_listado => params[:update_listado] }  if params[:pisa_formulario]
        #end
      end
    end
  end

  # Carga espacios y etiquetas
  def carga_espacios_etiquetas
    espacio =  Espacio.find_by_id(@espacio_id)
    if params[:seccion] == "documentos"
      condiciones_espacio = "proyecto_id is NULL and nombre != 'Proyectos' and nombre != 'Plantillas exportación' 
                             and definicion_espacio_proyecto_id is NULL and definicion_espacio_proyecto is NULL " 
      @espacios = Espacio.find(:all, :conditions => condiciones_espacio).collect {|e| [e.ruta + " / " + e.nombre, e.id] unless e.definicion_espacio_proyecto || e.definicion_espacio_pais || e.definicion_espacio_financiador || e.definicion_espacio_agente}.compact!
      @espacios.sort! { |x,y| x[0] <=> y[0] } if @espacios
      @espacios = [] unless @espacios
    elsif params[:seccion] == "proyectos" or params[:seccion] == "agentes"
      @objeto = @proyecto ? @proyecto : @agente
      @espacios = @objeto.espacio.espacio_hijo
      @espacios = @espacios + [@objeto.espacio] - [espacio] if @espacio_id
      @espacios = @espacios.collect {|e| [e.nombre , e.id]}
    end
    @etiquetas = @etiquetas_exportacion = Etiqueta.find(:all, :conditions => ["etiqueta.nombre LIKE ?", _("Exportacion") + " %"])
    if espacio && espacio.proyecto_del_espacio
      @etiquetas = Etiqueta.find(:all, :conditions => "tipo = 'proyecto' or tipo ='comunes'") - @etiquetas_exportacion
    elsif params[:seccion] == "documentos" or params[:seccion] == "agentes"
      @etiquetas = Etiqueta.find(:all, :conditions => {"tipo" => "comunes"}) - @etiquetas_exportacion
    end
    @etiquetas_documento = @documento.etiqueta.collect{ |e| e.id }
  end



	# modifica o crea
  def modificar_crear
    @documento = Documento.find_by_id(params[:id]) || Documento.new
    # Solo ponemos el usuario_id si no existe o el documento es nuevo
    params[:documento][:usuario_id] = @usuario_identificado.id if !params[:id] || !@documento.usuario_id

    # Todo esto habra que limpiarlo para eliminar vinculacion de documentos con proyectos y agentes (es necesario?)
    espacio_actual = Espacio.find_by_id(@espacio_id) if params[:tipo].nil? || params[:tipo] == ""
    # Si estamos en documentacion general y un espacio de proyecto, le metemos proyecto_id
    if (espacio_actual && params[:documento] && params[:seccion] == "documentos" && espacio_actual.proyecto_del_espacio)
      params[:documento][:proyecto_id] = espacio_actual.proyecto_del_espacio.id
    # Si estamos en un espacio de agente y no hay agente_id, se lo metemos
    elsif (espacio_actual && params[:documento] && params[:seccion] == "documentos" && espacio_actual.agente_del_espacio)
      params[:documento][:agente_id] = espacio_actual.agente_del_espacio.id
    end
    
    # Actualiza el objeto
    @documento.update_attributes params[:documento]
    if params[:tipo] && params[:tipo] != ""
      # Si estamos trabajando en las fuentes de verificacion, gastos, o transferencias vinculamos el documento a la fuente de verificacion
      #tipo, objeto_id = obtiene_tipo_y_objeto
      logger.info ">>>>>>>>>>>>>>> Asociamos Documento a un objeto " + params[:tipo] + " <<<<<<<<<<<<<<<<<<"
      @objeto = eval( params[:tipo] ).find_by_id(params[:objeto_id].to_i)
      @objeto.documento << @documento unless @objeto.nil? || @objeto.documento.find_by_id(@documento.id)
      #Cargamos los documentos del objeto para refrescar el listado
      @documentos = @objeto.documento
    else
      # Si estamos trabajando en documentos por espacio
      documento_x_espacio = DocumentoXEspacio.find_or_create_by_documento_id_and_espacio_id @documento.id, params[:documento_x_espacio][:espacio_id]
      cambio_de_espacio = false
      # Para cambiar un documento de espacio dentro de proyectos
      if params[:cambiar] and params[:cambiar][:espacio] == "1" and params[:cambiar][:espacio_id] != ""
        (DocumentoXEspacio.find_by_documento_id_and_espacio_id @documento.id, @espacio_id).destroy
        DocumentoXEspacio.create :documento_id => @documento.id, :espacio_id => params[:cambiar][:espacio_id]
        cambio_de_espacio = true if @espacio_id.to_s != params[:cambiar][:espacio_id] 
      end
    end
    #Finalmente guardamos las etiquetas vinculadas al documento
    @documento.etiqueta.clear
    @documento.etiqueta_ids = params["etiqueta"].to_a.collect {|p| p[0] if p[1] == "1"} if @documento.errors.empty?
    if @documento.errors.empty?
      responds_to_parent do
        # Si estamos en el listado de documentos
        unless params[:tipo] && params[:tipo] != ""
          i = params[:i] || params[:url][:i]
          render :update do |page|
            if cambio_de_espacio 
              page.remove "formularioinline"
              page.remove "formulariofondo"
              page.remove "formulariocontenedor"
              page.eliminar(:update => params[:update], :mensaje => { :errors => @documento.errors })
            elsif params[:id]
              page.modificar(:update => params[:update], :partial => "documento", :mensaje => { :errors => @documento.errors })
            else
              page.show "nuevos_documentos"
              page.modificar(:update => "documento_nuevo_" + params[:i], :partial => "nuevo_documento", :mensaje => { :errors => @documento.errors })
              page.replace "anadir", :inline => "<%= anadir(:url => {:action => 'asociar_o_nuevo', :i => (params[:i].to_i + 1).to_s, :update => 'formulario_anadir'}) %>"
            end
          end
        # Si es una vinculacion a un documento recargamos todo el listado
        else
          # Para otros objetos (en sublistado)
          render(:update){ |page|  page.modificar(:update_listado => params[:update_listado], :partial => "listado_asociados", :locals => {:objeto_id => params[:objeto_id], :tipo => params[:tipo] }, :mensaje => { :errors => @documento.errors}, :tipo_update => "sublistado") } 
        end
      end
    else
      # Si hay datos que fallan se vuelve al formulario
      carga_espacios_etiquetas
      responds_to_parent do
        render(:update){ |page|  page.recargar_formulario :partial => "formulario", :mensaje => { :errors => @documento.errors } }
      end
    end
  end    

  def modificar_crear_imagen_datos_proyecto
    # Solo sube el documento si hay un proyecto
    if @proyecto
      documento = Documento.new
      params[:documento] = Hash.new
      params[:documento][:adjunto] = params[:file]
      params[:documento][:usuario_id] = @usuario_identificado.id
      params[:documento][:proyecto_id] = @proyecto.id

      # Actualiza el objeto
      documento.update_attributes params[:documento]

      if documento.errors.empty?
        # Lo mete en el espacio
        espacio = @proyecto.espacio
        documento_x_espacio = DocumentoXEspacio.find_or_create_by_documento_id_and_espacio_id(documento.id, espacio.id)
        # Y lo muestra de vuelta
        render json: {
          image: {
            url: url_for(:only_path => false, :action => 'descargar', :controller => "documento", :id => documento)
          }
        }, content_type: "text/html"
      end
    end
  end

	# elimina documento
  def eliminar
    @documento = Documento.find(params[:id])
    @espacio =  Espacio.find(@espacio_id) if @espacio_id

    # Averiguamos si el documento esta vinculado y tratamos de borrar el original, lo rechaza 
    vinculado = @espacio ? (@documento.espacio.size > 1 && @documento.espacio.first == @espacio) : true 

    # Segun donde estemos podremos o no borrar
    # En ejecucion (documentos de gastos, transferencias o fuentes de verificacion de proyecto o de agente) podremos siempre borrar (no hay espacio asociado)
    if params[:menu] == "ejecucion" || params[:menu] == "gasto_agente" || params[:menu] == "movimientos_agente"
      @objeto = eval( params[:tipo] ).find_by_id(params[:objeto_id])
      # desvinculamos el documento del objeto
      @objeto.documento.delete(@documento)
      # Si el documento no esta vinculado a ningun otro objeto lo borramos
      @documento.destroy if @documento.send(params[:tipo].tableize).empty?
      # Y decimos que no esta realmente vinculado
      vinculado = false
    # En el resto (documentacion general, de proyectos y de agentes) borramos si no está vinculado
    elsif !vinculado
      @espacio.documento.delete(@documento)
      # Si el documento no esta vinculado a ningun otro objeto lo borramos
      @documento.destroy if @documento.espacio.reload.empty?
    end

    render :update do |page|  
        page.eliminar(:update => params[:update], :mensaje => { :errors => @documento.errors, :eliminar => true }) unless vinculado
        page.mensaje_informacion(params[:update], @documento.espacio.inject((_("No se puede borrar el documento por que se encuentra vinculado en algun espacio de documentación") + ":<br>").html_safe){|m,a| m += h(a.ruta + " / " + a.nombre) + "<br>".html_safe}, :tipo_mensaje => "mensajefallo") if vinculado
    end
  end

  def descargar
    @documento = Documento.find(params[:id])
      # file = File.new(@document.document.path, "r")
      # send_data(file.read,
      #  :filename     =>  @document.document_file_name,
      #  :type         =>  @document.document_content_type,
      #  :disposition  =>  'inline') 

    # Antes de descargar, comprobar que exista el fichero y marcar el error si no esta
    if File.exists?(@documento.adjunto.path)       
      send_file @documento.adjunto.path, :type => @documento.adjunto_content_type, :disposition => 'inline'
    else
      msg_error _("No se pudo encontrar el documento. Contacte con el administrador del sistema.")
      redirect_to :action => 'listado'
    end
  end


   # muestra los documentos vinculados a un objeto
  def listado_asociados
    @documentos = []
    @objeto = eval( params[:tipo] ).find_by_id(params[:objeto_id])
    @documentos = @objeto.documento if @objeto
    render :update do |page|
      page.replace_html params[:update], :partial => "listado_asociados", :locals => { :objeto_id => params[:objeto_id], :tipo => params[:tipo], :documentos => @documentos, :update_listado => params[:update]  }
    end
  end

  def asociar_o_nuevo
    render :update do |page|
      page.formulario :partial => "formulario_asociar_o_nuevo", :tipo => params[:tipo], :objeto_id => params[:objeto_id], :update => params[:update] 
    end
  end

  def asociar_documentos
    # Obtiene los documentos a vincular
    obtiene_documentos_a_vincular

    # Si pedimos asociar a algun tipo (fuente, gasto o transferencia), presentamos el formulario limitado
    if params[:tipo]
      render(:update) { |page| page.replace 'formulariocontenedor', :partial => "formulario_asociar" }
    # Si vamos a asociar sobre un espacio, presentamos el listado de navegacion
    else
      render(:update) { |page| page.replace_html 'formulariocontenido', :partial => "listado_asociar" }
    end
  end

  #Asociamos un documento a un objeto o a un espacio
  def asociar
    if params[:tipo] and params[:tipo] != ""
      # Si estamos asociando a un objeto asociamos el documento al objeto
      @objeto = eval( params[:tipo] ).find_by_id(params[:objeto_id])
      @relacion = eval(params[:tipo] + "XDocumento").create( params[:tipo].tableize + "_id" => params[:objeto_id], "documento_id" => params[:documento][:id] )
      @documentos = @objeto.documento if @objeto
    else
      # Si no hay objeto asociamos el documento al espacio de navegacion
      @documento = Documento.find(params[:documento][:id])
      espacio_seleccionado_id = session[:espacio_proyecto_seleccionado] if @proyecto 
      espacio_seleccionado_id = session[:espacio_agente_seleccionado] if @agente
      espacio_seleccionado_id = session[:espacio_seleccionado] unless @proyecto or @agente
      @relacion = DocumentoXEspacio.create(:espacio_id => espacio_seleccionado_id, :documento_id => @documento.id )
    end
    if @relacion.errors.empty?
      if params[:tipo]    
        # Si la asociacion es a un objeto refrescamos el listado de documentos del objeto
        render(:update){ |page|  page.modificar(:update_listado => params[:update_listado], :partial => "listado_asociados", :locals => {:objeto_id => params[:objeto_id], :tipo => params[:tipo]}, :mensaje => { :errors => @relacion.errors}, :tipo_update => "sublistado") }
      elsif
        # Si el documento se asocia a un espacio lo mostramos en la zona de nuevos documentos
        render :update do |page|
          page.show "nuevos_documentos"
          page.modificar(:update => "documento_nuevo_" + params[:i], :partial => "nuevo_documento", :mensaje => { :errors => @documento.errors })
          page.replace "anadir", :inline => "<%= anadir(:url => {:action => 'asociar_o_nuevo', :i => (params[:i].to_i + 1).to_s, :update => 'formulario_anadir'}) %>"
        end
      end
    else
      # Si hay datos que fallan se vuelve al formulario
      obtiene_documentos_a_vincular
      if params[:tipo]
        render(:update) { |page|  page.recargar_formulario :partial => "formulario_asociar", :mensaje => { :errors => @relacion.errors } }
      else
        render(:update) { |page| page.mensaje_actualizacion "formularioinline", { :errors => @relacion.errors } }
      end
    end
  end

 private

  def obtiene_espacios_y_documentos
    #Si estamos en administracion forzamos siempre estar en el mismo espacio
    @espacio = Espacio.find_by_id(@espacio_id) if @espacio_id
    # Salvo que estemos administrando espacios, evita que un no autorizado entre en el espacio
    @espacio = nil if @espacio && params[:seccion] != "administracion" && @espacio.usuario_no_permitido(@usuario_identificado)
    @documentos = @espacio ? @espacio.reload.documento : [] 
    condiciones = ['espacio_padre_id = ?', @espacio.id ] if @espacio
    condiciones = '(espacio_padre_id is NULL or espacio_padre_id = "0") and
                   (definicion_espacio_proyecto is NULL or definicion_espacio_proyecto = "0") and 
                   (definicion_espacio_agente is NULL or definicion_espacio_agente = "0") and
                   (definicion_espacio_socia is NULL or definicion_espacio_socia = "0") and
                   (definicion_espacio_financiador is NULL or definicion_espacio_financiador = "0") and
                   (definicion_espacio_pais is NULL or definicion_espacio_pais = "0")' unless @espacio
    espacios = Espacio.find(:all, :order => 'nombre', :conditions => condiciones)
    #Quitamos los espacios ocultos o aquellos sobre los que no se tienen permisos especificos
    espacios.reject! {|e| (e.usuario_no_permitido @usuario_identificado)}
    # Construye el hash para manejar espacios
    @espacios = (@espacio || @espacio_id == "0") ? [] : [{:nombre => _("Implementadores"), :action => 'implementador', :descripcion => _("Espacio de delegaciones y socias locales asignadas")}]
    @espacios = espacios.inject(@espacios) {|m,a| m.push({:nombre => a.nombre, :id => a.id, :action => 'seleccionar_espacio', :descripcion => a.descripcion}) }
  end

  def obtiene_documentos_a_vincular
    @objeto = @proyecto ? @proyecto : @gasto
    if params[:tipo] == "FuenteVerificacion"
      #Si estamos asociando a una fuente de verificacion
      documentos_fuente =  FuenteVerificacion.find(params[:objeto_id]).documento
      documentos = (FuenteVerificacionXDocumento.find(:all, :include => [:documento], :conditions => {"documento.proyecto_id" => @proyecto.id}).collect {|d| d.documento}).uniq
      documentos = documentos - documentos_fuente
      @documentos = documentos.collect {|d| [d.adjunto_file_name, d.id]}
    elsif params[:tipo] == "Gasto"
      # Si estamos asociando a un gasto
      gasto = Gasto.find params[:objeto_id]
      gastos = Gasto.find_all_by_agente_id(@agente.id) if params[:seccion] == "agentes"
      gastos = Gasto.find(:all, :include => [:gasto_x_proyecto], :conditions => {"gasto_x_proyecto.proyecto_id" => @proyecto.id}) if params[:seccion] == "proyectos"
      documentos = (gastos.inject(Array.new){|sum, n| sum + n.documento} - gasto.documento).uniq
      @documentos = documentos.collect {|d| [d.adjunto_file_name, d.id]}
    elsif params[:tipo] == "Transferencia"
      # Si estamos asociando a un gasto
      transferencia = Transferencia.find_by_id params[:objeto_id]
      if @proyecto
        transferencias = Transferencia.find_all_by_proyecto_id @proyecto.id if params[:seccion] == "proyectos"
      elsif @agente
        condiciones = "(libro.agente_id = " + @agente.id.to_s + " OR libro_destinos_transferencia.agente_id = " + @agente.id.to_s + ")"
        transferencias = Transferencia.find(:all, :include => [:libro_origen, :libro_destino], :conditions => condiciones)
      end
      documentos = (transferencias.inject(Array.new){|sum, n| sum + n.documento} - transferencia.documento).uniq
      @documentos = documentos.collect {|d| [d.adjunto_file_name, d.id]}
    else
      # Si estamos asociando dentro de otro espacio
      # Primero recogemos los documentos del propio espacio
      @espacio_donde_vincular = Espacio.find_by_id(@espacio_id)
      documentos_a_excluir = @espacio_donde_vincular ? @espacio_donde_vincular.reload.documento : []
      # Obtenemos documentos y espacios del seleccionado
      @espacio_id = params[:espacio_id] if params[:espacio_id]
      obtiene_espacios_y_documentos
      # Y eliminamos los ya contenidos en el espacio
      @documentos = @documentos - documentos_a_excluir
    end
  end

end
