# encoding: UTF-8
class NuevosEspaciosAgentes < ActiveRecord::Migration
  def up
    # Crea atributo para evitar modificacion de ciertos espacios
    add_column :espacio, :modificable, :boolean, :default => true
    add_column :espacio, :definicion_espacio_financiador, :boolean, :default => false
    add_column :espacio, :definicion_espacio_financiador_id, :integer
    add_column :espacio, :pais_id, :integer
    add_column :espacio, :definicion_espacio_pais, :boolean, :default => false
    add_column :espacio, :definicion_espacio_pais_id, :integer

    # Recarga la informacion de Espacio con los cambios anteriores
    Espacio.reset_column_information

    # Modifica espacios fijos ya existentes (o los crea)
    ["Proyectos","Plantillas Exportación"].each do |nombre|
      espacio = Espacio.find_by_nombre(nombre)
      if espacio
        espacio.update_attribute(:modificable, false)
      else
        logger.info "ERROR: No existe el espacio con nombre " + nombre
        puts "------------> ERROR: No existe el espacio con nombre " + nombre
      end
    end

    # Genera el espacio de financiadores
    # Primero comprueba que no exista
    if financiadores_old = Espacio.find_by_nombre_and_espacio_padre_id("Financiadores",nil)
      financiadores_old.update_attributes(:nombre => "Financiadores_espacio_viejo") if financiadores_old.espacio_padre_id.nil?
    end
    #e_financ = Espacio.create :nombre => "Financiadores", :modificable => false, :descripcion => "Espacio de financiadores."
    e_financ = Espacio.find_by_nombre("Financiadores")
    # Recorre todos los financiadores puros, metiendolos en el espacio_financiador
    Agente.all(:conditions => {:financiador => true}).each do |financ|
      # Comprueba que no exista previamente
      esp_financiador = Espacio.find_by_nombre_and_espacio_padre_id(financ.nombre, financiadores_old.id) if financiadores_old
      if esp_financiador
        esp_financiador.update_attributes(:agente_id => financ.id, :espacio_padre_id => e_financ.id)
      else
        financ.update_attribute(:nombre, financ.nombre) unless financ.implementador
      end
    end if e_financ

    # Genera el espacio de paises
    # primero comprueba que no exista antes
    if paises_old = Espacio.find_by_nombre_and_espacio_padre_id("Paises",nil)
      paises_old.update_attributes(:nombre => "Paises_espacio_viejo") if paises_old.espacio_padre_id.nil?
    end
    # Y luego lo crea
    espacio_pais = Espacio.find_by_nombre_and_espacio_padre_id("Paises",nil)
    #espacio_pais = Espacio.create :nombre => "Paises", :modificable => false, :descripcion => "Espacio de los paises."
    # Y recorre todos los paises creando su espacio
    Pais.all.each do |p|
      p.update_attribute(:nombre, p.nombre)
      espacio = p.reload.espacio 
      if espacio
        # Y metiendo los agentes en el
        p.agente.all(:conditions => {:implementador => true}).each do |a|
          a.update_attribute(:nombre, a.nombre)
        end
      else
        logger.info "ERROR: No se pudo crear el espacio para el pais " + p.nombre + ": " + p.errors.messages.to_s
        puts "------------> ERROR: No se pudo crear el espacio para el pais " + p.nombre + ": " + p.errors.messages.to_s
      end
    end if espacio_pais
  end

  def down
    espacio_agentes = Espacio.find_or_create_by_nombre("Agentes")
    Agente.all(:conditions => {:financiador => false, :implementador => true}).each do |impl|
      espacio = impl.espacio
      espacio.update_attribute(:espacio_padre_id, espacio_agentes.id) if espacio 
    end if espacio_agentes
    remove_column :espacio, :modificable
    remove_column :espacio, :pais_id
    remove_column :espacio, :definicion_espacio_pais
    remove_column :espacio, :definicion_espacio_pais_id
    remove_column :espacio, :definicion_espacio_financiador
    remove_column :espacio, :definicion_espacio_financiador_id
  end
end
